/*
 * downloadItem
 */

/**
 * Constructor for a download item.
 * @param (Object) data data object from chrome.downloads API 
 */
function DownloadItem(data) {
	var item = this;
	for (var prop in data) {
		item[prop] = data[prop];
	}
	item.startTime = new Date(item.startTime);

	item.div = this.createMainHTMLElement();
	item.div.item = item;
	item.dateId = item.startTime.getFullYear().toString() + item.startTime.getMonth().toString() + item.startTime.getDate().toString();

	var itemGroup = document.getElementById('itemgroup' + item.dateId)
	if (!itemGroup) {
		itemGroup = item.downloadItemDateGroup();
		document.getElementById('items').appendChild(itemGroup);
	}
	var itemsDiv = itemGroup.childNodes[1];
	item.itemGroup = itemGroup;
	item.moreBox = item.div.childNodes[1];

	var lenItemsDiv = itemsDiv.childNodes.length;
	/* 
	 * When the history is first initialized all the items go into the append case, because the items
	 * are already ordered by startTime descending. If the list is currently open and a download
	 * starts it should be the most recent, and thus has to be inserted before.
	 */
	if (lenItemsDiv === 0 ||
		(item.startTime.getTime() < itemsDiv.childNodes[lenItemsDiv - 1].item.startTime.getTime())) {
		itemsDiv.appendChild(item.div);
	} else {
		itemsDiv.insertBefore(item.div, itemsDiv.childNodes[0]);
	}

	item.getElement('download-item-filelink').onclick = function() {
		item.open();
		return false;
	};
	item.getElement('download-item-filelink').ondragstart = function() {
		item.drag();
		return false;
	};
	item.getElement('download-item-cancel').onclick = function() {
		item.cancel();
		return false;
	};
	item.getElement('download-item-show-in-files').onclick = function() {
		item.show();
		return false;
	};
	item.getElement('download-item-close-button').onclick = function() {
		item.erase();
		return false;
	};

	item.getElement('download-item-more-button').onclick = function(e) {
		e.stopPropagation();
		item.toggleMoreBoxVisibility();
		return false;
	}
	item.getElement('download-item-more-box-open').onclick = function() {
		item.open();
		item.getElement('download-item-more-box').hidden = true;
		return false;
	};
	item.getElement('download-item-more-box-show-in-files').onclick = function() {
		item.show();
		item.getElement('download-item-more-box').hidden = true;
		return false;
	};
	item.getElement('download-item-more-box-remove').onclick = function() {
		item.erase();
		document.removeEventListener('click', item.onClickOutsideMoreBox);
		return false;
	};

	item.onClickOutsideMoreBox = function(e) {
		console.log("click outside");
		var moreBox = item.getElement('download-item-more-box');
		moreBox.hidden = true;
		document.removeEventListener('click', item.onClickOutsideMoreBox);
		moreBox.removeEventListener('click', item.onClickInsideMoreBox);
	}

	item.onClickInsideMoreBox = (e) => e.stopPropagation();
	item.render();
}

/**
 * Render function for the Download item.
 * This renders the item according to its state.
 */
DownloadItem.prototype.render = function() {
	var item = this;
	var now = new Date();
	var inProgress = (item.state === 'in_progress')
	var openable = (item.state !== 'interrupted') && item.exists && !item.deleted;
	var hasError = (item.error !== undefined);

	item.startTime = new Date(item.startTime);

	if (item.filename) {
		item.basename = item.filename.substring(Math.max(
			item.filename.lastIndexOf('\\'),
			item.filename.lastIndexOf('/')) + 1);
	}
	if (item.estimatedEndTime) {
		item.estimatedEndTime = new Date(item.estimatedEndTime);
	}
	if (item.endTime) {
		item.endTime = new Date(item.endTime);
	}
	if (item.filename && !item.iconUrl) {
		chrome.downloads.getFileIcon(
			item.id,
			{'size': 32},
			function(iconUrl) {
				if (iconUrl) {
					item.iconUrl = iconUrl;
					item.getElement('download-item-icon').src = iconUrl;
				}
		});
	}

	item.getElement('download-item-filename').innerHTML = item.basename;
	item.getElement('download-item-filelink').innerHTML = item.basename;
	item.getElement('download-item-filename').style.display = (
		(openable && !inProgress) ? 'none' : 'inline-block');
	item.getElement('download-item-filename').style.color = (
		(hasError || !openable) ? 'grey' : 'black');
	item.getElement('download-item-filelink').style.display = (
		(openable && !inProgress) ? 'inline-block' : 'none');
	item.getElement('download-item-show-in-files').style.display = (
		(openable && !inProgress) ? 'inline-block' : 'none');
	item.getElement('download-item-cancel').style.display = (
		inProgress ? 'inline-block' : 'none');
	item.getElement('download-item-progress').style.display = (
		inProgress ? 'none' : 'inline-block');
	item.getElement('download-item-time-left').style.display = (
		inProgress ? 'inline-block' : 'none');
	item.getElement('download-item-time-left').innerHTML = (
		item.estimatedEndTime ?
			DownloadManager.formatTimeLeft(item.estimatedEndTime.getTime() - now.getTime())
			: gt.gettext('unknown'));
	item.getElement('download-item-start-time').innerHTML =
		item.startTime.toLocaleString(window.navigator.language,
				{hour: 'numeric', minute: 'numeric'});
	item.getElement('download-item-more-button').style.display = (
		inProgress ? 'none' :'inline-block');
	item.getElement('download-item-progress').style.display = (
		inProgress ? 'inline-block' : 'none');
	if (item.totalBytes && (item.state !== 'complete')) {
		item.getElement('download-item-progress-text').innerHTML = (
			DownloadManager.formatBytes(item.bytesReceived) + '/' +
			DownloadManager.formatBytes(item.totalBytes));
		item.getElement('download-item-progress-bar').style.width = parseInt(
				100 * item.bytesReceived / item.totalBytes) + '%';
	}
	item.getElement('download-item-error').style.display = (
		(hasError || !openable) ? 'inline-block' : 'none');
	item.getElement('download-item-error').innerHTML = (
		item.error ? DownloadManager.errorMessages[item.error] : gt.gettext('Removed'));
	item.getElement('download-item-filesize').style.display = (
		(openable && !inProgress) ? 'inline-block' : 'none');
	item.getElement('download-item-filesize').innerHTML =
		'(' + DownloadManager.formatBytes(item.bytesReceived) + ')';
	item.getElement('download-item-more-button').style.display = (
		(openable && !inProgress) ? 'inline-block' : 'none');

};

/**
 * Get a subelement by its className
 * @param (String) name className of the element to get 
 */
DownloadItem.prototype.getElement = function(name) {
	return document.querySelector('#item' + this.id + ' .' + name);
};

/**
 * Is called when a Download has changed its state.
 * @param (Object) delta object containing all changed attributes of a Download item
 */
DownloadItem.prototype.applyChanges = function(delta) {
	for (var key in delta) {
		if (key != 'id') {
			this[key] = delta[key].current;
		}
	}
	this.render();
	if ((this.state == 'in_progress') && !this.paused) {
		DownloadManager.startPollingProgress();
	}
};

/**
 * Function to erase the Download item.
 * Erasing means removal from the list of downloaded items.
 * The downloaded file will stay on the file system.
 */
DownloadItem.prototype.erase = function() {
	chrome.downloads.erase({id: this.id});
	document.getElementById('item' + this.id).remove();
	if (this.itemGroup.childNodes[1].childNodes.length === 0) {
		this.itemGroup.remove();
	}
};

/**
 * Handler when a Download item gets dragged.
 * Downloads can be dragged out from the filelink in the history popup to the filesystem.
 */
DownloadItem.prototype.drag = function() {
	chrome.downloads.drag(this.id);
};

/**
 * Handler when a Download item should be shown in its directory.
 */
DownloadItem.prototype.show = function() {
	chrome.downloads.show(this.id);
};

/**
 * Handler when a Download item should be opened with the OS default program.
 */
DownloadItem.prototype.open = function() {
	if (this.state == 'complete') {
		chrome.downloads.open(this.id);
		return;
	}
};

/**
 * Cancel the download of a Download item.
 */
DownloadItem.prototype.cancel = function() {
	chrome.downloads.cancel(this.id);
};

/**
 * Show or hide the more box of a Download item.
 */
DownloadItem.prototype.toggleMoreBoxVisibility = function() {
	var moreBox = this.getElement('download-item-more-box');
	if (!this.moreBox.hidden) {
		this.moreBox.hidden = true;
	} else {
		this.moreBox.hidden = false;
		this.moreBox.style.position = 'absolute';
		this.moreBox.style.top = (this.div.offsetTop + this.div.offsetHeight - 10) + 'px';
		this.moreBox.style.left = (this.div.offsetLeft + this.div.offsetWidth - 110) + 'px';
		document.addEventListener('click', this.onClickOutsideMoreBox);
		this.moreBox.addEventListener('click', this.onClickInsideMoreBox);
	}
};

/**
 * Create the HTML for a Download item
 * @return {Object} the HTML Element for the Download item.
 */
DownloadItem.prototype.createMainHTMLElement = function() {
	var item = this.createEl('table', 'download-item', 'item' + this.id);
	item.innerHTML = `
		<tbody>
			<tr>
				<td class="download-item-icon-cell" rowspan="2">
					<img class="download-item-icon" src="../../../resources/images/icon_empty.png">
				</td>
				<td class="download-item-details-top">
					<a class="download-item-filelink" href="#"></a>
					<div class="download-item-filename"></div>
					<div class="download-item-filesize"></div>
					<button class="download-item-close-button"></button>
					<div class="download-item-start-time"></div>
				</td>
			</tr>
			<tr>
				<td class="download-item-details-bottom">
					<a class="download-item-show-in-files" href="#">${gt.gettext('Show in files')}</a>
					<div class="download-item-error"></div>
					<div class="download-item-time-left"></div>
					<div class="download-item-progress">
						<div class="download-item-progress-bar"></div>
						<div class="download-item-progress-text"></div>
					</div>
					<a class="download-item-cancel" href="#">${gt.gettext('Cancel')}</a>
					<button class="download-item-more-button"></button>
				</td>
			</tr>
		</tbody>
		<div class="download-item-more-box" hidden>
			<a class="download-item-more-box-open" href="#">${gt.gettext('Open')}</a>
			<a class="download-item-more-box-show-in-files" href="#">${gt.gettext('Show in files')}</a>
			<a class="download-item-more-box-remove" href="#">${gt.gettext('Remove from list')}</a>
		</div>
	`.replace(/>\s+</g, "><").trim();
	return item;
}

/**
 * Create the HTML for a Download item date group.
 * @return {Object} the HTML Element for the Download item date group.
 */
DownloadItem.prototype.downloadItemDateGroup = function() {
	var dateString;
	if (this.startTime.isToday()) {
		dateString = gt.gettext('Today');
	} else if (this.startTime.isYesterday()) {
		dateString = gt.gettext('Yesterday');
	} else {
		dateString = this.startTime.toLocaleString(window.navigator.language,
			{day: 'numeric', month: 'long', year: 'numeric'});
	}

	var itemGroup = this.createEl('div', 'download-item-group', 'itemgroup' + this.dateId);
	itemGroup.innerHTML = `
		<div class="download-item-group-header">${dateString}</div>
		<div class="download-item-group-itemdiv"></div>
	`.replace(/>\s+</g, "><").trim();

	return itemGroup;
}

/**
 * Create simple HTML elements from parameters
 * @param {String} type tag of the Element to create
 * @param {String} cls className of the Element to create (optional)
 * @param {String} id id of the Element to create (optional)
 * @return {Object} HTML element
 */
DownloadItem.prototype.createEl = function(type, cls, id) {
	var el = document.createElement(type);
	if (cls) {
		el.className = cls;
	}
	if (id) {
		el.id = id;
	}
	return el;
}